Package: HiCParser
Type: Package
Title: Parser for HiC data in R
Date: 2025-07-22
Version: 1.4.0
URL: https://github.com/emaigne/HiCParser
BugReports: https://github.com/emaigne/HiCParser/issues
Authors@R: c(
    person("Zytnicki", "Matthias", email = "matthias.zytnicki@inrae.fr", 
      role = c("aut")),
    person("Maigné", "Élise", email = "elise.maigne@inrae.fr", 
      role = c("aut", "cre")))
Description: This package is a parser to import HiC data into R. It
        accepts several type of data: tabular files, Cooler `.cool` or
        `.mcool` files, Juicer `.hic` files or HiC-Pro `.matrix` and
        `.bed` files. The HiC data can be several files, for several
        replicates and conditions. The data is formated in an
        InteractionSet object.
License: LGPL
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
biocViews: Software, HiC, DataImport
Imports: data.table, InteractionSet, GenomicRanges,
        SummarizedExperiment, Rcpp (>= 1.0.12), S4Vectors, gtools,
        pbapply, BiocGenerics, Seqinfo
LinkingTo: Rcpp
Suggests: rhdf5, BiocStyle, knitr, sessioninfo, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:04:28 UTC
RemoteUrl: https://github.com/bioc/HiCParser
RemoteRef: RELEASE_3_23
RemoteSha: 688ecf5636fa7a362eb849b3de5805633ffebbd4
NeedsCompilation: yes
Packaged: 2026-04-29 22:17:47 UTC; root
Author: Zytnicki Matthias [aut],
  Maigné Élise [aut, cre]
Maintainer: Maigné Élise <elise.maigne@inrae.fr>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 22:42:57 UTC; unix

# Generated by roxygen2: do not edit by hand

export(mergeInteractionSet)
export(parseCool)
export(parseHiC)
export(parseHiCPro)
export(parseTabular)
import(data.table)
importFrom(BiocGenerics,cbind)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,match)
importFrom(GenomicRanges,union)
importFrom(InteractionSet,GInteractions)
importFrom(InteractionSet,InteractionSet)
importFrom(InteractionSet,anchors)
importFrom(InteractionSet,interactions)
importFrom(InteractionSet,intrachr)
importFrom(InteractionSet,reduceRegions)
importFrom(InteractionSet,regions)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(data.table,fread)
importFrom(data.table,merge.data.table)
importFrom(data.table,setalloccol)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,shift)
importFrom(gtools,mixedsort)
importFrom(pbapply,pbmapply)
useDynLib(HiCParser)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# HiCParser

<!-- badges: start -->

[![GitHub
issues](https://img.shields.io/github/issues/emaigne/HiCParser)](https://github.com/emaigne/HiCParser/issues)
[![GitHub
pulls](https://img.shields.io/github/issues-pr/emaigne/HiCParser)](https://github.com/emaigne/HiCParser/pulls)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![Bioc release
status](http://www.bioconductor.org/shields/build/release/bioc/HiCParser.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/HiCParser)
[![Bioc devel
status](http://www.bioconductor.org/shields/build/devel/bioc/HiCParser.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/HiCParser)
[![Bioc downloads
rank](https://bioconductor.org/shields/downloads/release/HiCParser.svg)](http://bioconductor.org/packages/stats/bioc/HiCParser/)
[![Bioc
support](https://bioconductor.org/shields/posts/HiCParser.svg)](https://support.bioconductor.org/tag/HiCParser)
[![Bioc
history](https://bioconductor.org/shields/years-in-bioc/HiCParser.svg)](https://bioconductor.org/packages/release/bioc/html/HiCParser.html#since)
[![Bioc last
commit](https://bioconductor.org/shields/lastcommit/devel/bioc/HiCParser.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/HiCParser/)
[![Bioc
dependencies](https://bioconductor.org/shields/dependencies/release/HiCParser.svg)](https://bioconductor.org/packages/release/bioc/html/HiCParser.html#since)
[![R-CMD-check-bioc](https://github.com/emaigne/HiCParser/actions/workflows/check-bioc.yml/badge.svg)](https://github.com/emaigne/HiCParser/actions/workflows/check-bioc.yml)
[![Codecov test
coverage](https://codecov.io/gh/emaigne/HiCParser/branch/devel/graph/badge.svg)](https://app.codecov.io/gh/emaigne/HiCParser?branch=devel)
<!-- badges: end -->

The goal of `HiCParser` is to parse Hi-C data (`HiCParser` supports
serveral formats), and import them in R, as an `InteractionSet` object.

## Installation instructions

Get the latest stable `R` release from
[CRAN](http://cran.r-project.org/). Then install `HiCParser` from
[Bioconductor](http://bioconductor.org/) using the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("HiCParser")
```

And the development version from
[GitHub](https://github.com/emaigne/HiCParser) with:

``` r
BiocManager::install("emaigne/HiCParser")
```

Then load the package :

``` r
library("HiCParser")
```

## Supported formats

So far, `HiCParser` supports:

-   [cool and mcool](https://github.com/open2c/cooler) formats

-   [hic](https://github.com/aidenlab/hictools) format

-   [HiC-Pro](https://github.com/nservant/HiC-Pro) format

-   A tabular format, where

    -   the first column is named “chromosome”
    -   the second column is named “position 1” or “position.1”
    -   the third column is named “position 2” or “position.2”
    -   the fourth column is named “*x*.R*y*”, and *x* is the id of the
        condition (“1”, or “2”, usually), *y* is the id of the replicate
        (“1”, “2”, “3”, etc.); it should contain matrix counts
    -   the remaining columns are optional, and should be formatted like
        the fourth column

## Example

### hic format

We show here how to parse one hic format file.

``` r
hicFilePath <- system.file("extdata", "hicsample_21.hic", package = "HiCParser")
data <- parseHiC(
    paths = hicFilePath,
    binSize = 5000000,
    conditions = 1,
    replicates = 1
)
```

Note that a hic file can include several matrices, with different bin
sizes. This is why the bin size should be provided.

We show here how to parse several files (actually, the same file,
several times). We suppose here that we have 2 conditions, with 3
replicates for each condition.

``` r
data <- parseHiC(
    paths = rep(hicFilePath, 6),
    binSize = 5000000,
    conditions = rep(seq(2), each = 3),
    replicates = rep(seq(3), 2)
)
```

Currently, `HiCParser` supports the hic format up to the version 9.

### HiC-Pro format

A HiC-Pro file contains a matrix file, and a bed file. A different bed
file could be use for each matrix file, but the same can also be used.

``` r
matrixFilePath <- 
    system.file("extdata", "hicsample_21.matrix", package = "HiCParser")
bedFilePath <- 
    system.file("extdata", "hicsample_21.bed", package = "HiCParser")
data <- parseHiCPro(
    matrixPaths = rep(matrixFilePath, 6),
    bedPaths = bedFilePath,
    conditions = rep(seq(2), each = 3),
    replicates = rep(seq(3), 2)
)
```

### cool and mcool formats

Please note that the cool and mcool format store data in HDF5 format.
The [HDF5
package](https://bioconductor.org/packages/release/bioc/html/rhdf5.html)
is not included by default, because it requires a substantial time to be
compiled, and many users will not need the cool/mcool parser. So, in
order to use the cool/mcool parser, you should install the `rhdf5`
package.

The cool format include only one bin size.

``` r
if (!"rhdf5" %in% installed.packages()) {
    install.packages("rhdf5")
}
coolFilePath <- system.file("extdata",
    "hicsample_21.cool",
    package = "HiCParser"
)
data <- parseCool(
    paths = rep(coolFilePath, 6),
    conditions = rep(seq(2), each = 3),
    replicates = rep(seq(3), 2)
)
```

The mcool format may include several bin sizes. It is thus compulsory to
mention it. The same function is used for the cool/mcool formats.

``` r
mcoolFilePath <- system.file("extdata",
    "hicsample_21.mcool",
    package = "HiCParser"
)
data <- parseCool(
    paths = rep(mcoolFilePath, 6),
    binSize = 5000000,
    conditions = rep(seq(2), each = 3),
    replicates = rep(seq(3), 2)
)
```

## Tabular files

A tabular file is a tab-separated multi-replicate sparse matrix with a
header:

    chromosome    position 1    position 2    C1.R1    C1.R2    C1.R3    ...
    Y             1500000       7500000       145      184      72       ...

The number of interactions between `position 1` and `position 2` of
`chromosome` are reported in each `condition.replicate` column. There is
no limit to the number of conditions and replicates.

To load Hi-C data in this format:

``` r
hic.experiment <- parseTabular(
    system.file("extdata",
        "hicsample_21.tsv",
        package = "HiCParser"
    ),
    sep = "\t"
)
```

### Output

The output is a
[InteractionSet](https://bioconductor.org/packages/release/bioc/html/InteractionSet.html).
This object can store one or several samples. Please read the
[corresponding
vignette](https://bioconductor.org/packages/devel/bioc/vignettes/InteractionSet/inst/doc/interactions.html)
in order to known more about this format.

``` r
library("HiCParser")
hicFilePath <- system.file("extdata", "hicsample_21.hic", package = "HiCParser")
hic.experiment <- parseHiC(
    paths = rep(hicFilePath, 6),
    binSize = 5000000,
    conditions = rep(seq(2), each = 3),
    replicates = rep(seq(3), 2)
)
#> 
#> Parsing '/tmp/RtmpHFfOT6/temp_libpathc52a1c587e54/HiCParser/extdata/hicsample_21.hic'.
#> 
#> Parsing '/tmp/RtmpHFfOT6/temp_libpathc52a1c587e54/HiCParser/extdata/hicsample_21.hic'.
#> 
#> Parsing '/tmp/RtmpHFfOT6/temp_libpathc52a1c587e54/HiCParser/extdata/hicsample_21.hic'.
#> 
#> Parsing '/tmp/RtmpHFfOT6/temp_libpathc52a1c587e54/HiCParser/extdata/hicsample_21.hic'.
#> 
#> Parsing '/tmp/RtmpHFfOT6/temp_libpathc52a1c587e54/HiCParser/extdata/hicsample_21.hic'.
#> 
#> Parsing '/tmp/RtmpHFfOT6/temp_libpathc52a1c587e54/HiCParser/extdata/hicsample_21.hic'.
hic.experiment
#> class: InteractionSet 
#> dim: 44 6 
#> metadata(0):
#> assays(1): ''
#> rownames: NULL
#> rowData names(1): chromosome
#> colnames: NULL
#> colData names(2): condition replicate
#> type: StrictGInteractions
#> regions: 9
```

The conditions and replicates are reported in the `colData` slot :

``` r
SummarizedExperiment::colData(hic.experiment)
#> DataFrame with 6 rows and 2 columns
#>   condition replicate
#>   <integer> <integer>
#> 1         1         1
#> 2         1         2
#> 3         1         3
#> 4         2         1
#> 5         2         2
#> 6         2         3
```

They corresponds to columns of the `assays` matrix (containing
interactions values):

``` r
head(SummarizedExperiment::assay(hic.experiment))
#>      [,1] [,2] [,3] [,4] [,5] [,6]
#> [1,]   79   79   79   79   79   79
#> [2,]   22   22   22   22   22   22
#> [3,]    3    3    3    3    3    3
#> [4,]    1    1    1    1    1    1
#> [5,]    1    1    1    1    1    1
#> [6,]    2    2    2    2    2    2
```

The positions of interactions are in the `interactions` slot of the
object:

``` r
InteractionSet::interactions(hic.experiment)
#> StrictGInteractions object with 44 interactions and 1 metadata column:
#>        seqnames1           ranges1     seqnames2           ranges2 | chromosome
#>            <Rle>         <IRanges>         <Rle>         <IRanges> |      <Rle>
#>    [1]        21  5000001-10000000 ---        21  5000001-10000000 |         21
#>    [2]        21  5000001-10000000 ---        21 10000001-15000000 |         21
#>    [3]        21  5000001-10000000 ---        21 15000001-20000000 |         21
#>    [4]        21  5000001-10000000 ---        21 20000001-25000000 |         21
#>    [5]        21  5000001-10000000 ---        21 25000001-30000000 |         21
#>    ...       ...               ... ...       ...               ... .        ...
#>   [40]        21 35000001-40000000 ---        21 40000001-45000000 |         21
#>   [41]        21 35000001-40000000 ---        21 45000001-50000000 |         21
#>   [42]        21 40000001-45000000 ---        21 40000001-45000000 |         21
#>   [43]        21 40000001-45000000 ---        21 45000001-50000000 |         21
#>   [44]        21 45000001-50000000 ---        21 45000001-50000000 |         21
#>   -------
#>   regions: 9 ranges and 1 metadata column
#>   seqinfo: 1 sequence from an unspecified genome; no seqlengths
```

## Citation

Below is the citation output from using `citation('HiCParser')` in R.
Please run this yourself to check for any updates on how to cite
**HiCParser**.

To cite the ‘HiCParser’ HiCParser in a publication, use :

Maigné E, Zytnicki M (2024). *A multiple format Hi-C data parser*.
<doi:10.18129/B9.bioc.HiCParser>
<https://doi.org/10.18129/B9.bioc.HiCParser>,
<https://github.com/emaigne/HiCParser/HiCParser> - R package version
0.1.0, <http://www.bioconductor.org/packages/HiCParser>.

As a BibTeX entry :

    @Manual{hicparser,
      title = {A multiple format Hi-C data parser},
      author = {Elise Maigné and Matthias Zytnicki},
      year = {2024},
      url = {http://www.bioconductor.org/packages/HiCParser},
      note = {https://github.com/emaigne/HiCParser/HiCParser - R package version 0.1.0},
      doi = {10.18129/B9.bioc.HiCParser},
    }

Please note that the `HiCParser` was only made possible thanks to many
other R and bioinformatics software authors, which are cited either in
the vignettes and/or the paper(s) describing this package.

## Code of Conduct

Please note that the `HiCParser` project is released with a [Contributor
Code of Conduct](http://bioconductor.org/about/code-of-conduct/). By
contributing to this project, you agree to abide by its terms.

## Development tools

-   Continuous code testing is possible thanks to [GitHub
    actions](https://www.tidyverse.org/blog/2020/04/usethis-1-6-0/)
    through *[usethis](https://CRAN.R-project.org/package=usethis)*,
    *[remotes](https://CRAN.R-project.org/package=remotes)*, and
    *[rcmdcheck](https://CRAN.R-project.org/package=rcmdcheck)*
    customized to use [Bioconductor’s docker
    containers](https://www.bioconductor.org/help/docker/) and
    *[BiocCheck](https://bioconductor.org/packages/3.17/BiocCheck)*.
-   Code coverage assessment is possible thanks to
    [codecov](https://codecov.io/gh) and
    *[covr](https://CRAN.R-project.org/package=covr)*.
-   The [documentation website](http://emaigne.github.io/HiCParser) is
    automatically updated thanks to
    *[pkgdown](https://CRAN.R-project.org/package=pkgdown)*.
-   The code is styled automatically thanks to
    *[styler](https://CRAN.R-project.org/package=styler)*.
-   The documentation is formatted thanks to
    *[devtools](https://CRAN.R-project.org/package=devtools)* and
    *[roxygen2](https://CRAN.R-project.org/package=roxygen2)*.

For more details, check the `dev` directory.

This package was developed using
*[biocthis](https://bioconductor.org/packages/3.17/biocthis)*.

##############################################################################
# R code to generate inst/extdata/hicsample_21.txt file

# Reading matrix and bed files
mat21 <- data.table::fread("inst/extdata/hicsample_21.matrix")
bed21 <- data.table::fread("inst/extdata/hicsample_21.bed", drop=3)

# Setting colnames
setnames(mat21, c("index1", "index2", "1.R1"))
setnames(bed21, c("chromosome", "start", "index"))

# Merging data
tsv21 <- merge(bed21[,-1], mat21, by.x="index", by.y="index2", all.y = TRUE)
setnames(tsv21, "start", "position 2")
tsv21[,index := NULL]

tsv21 <- merge(bed21, tsv21, by.x="index", by.y="index1", all.y = TRUE)
setnames(tsv21, "start", "position 1")
tsv21[,index := NULL]

data.table::fwrite(tsv21, file="inst/extdata/hicsample_21.tsv", sep="\t")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL,
    warning = FALSE
)

## ----"citation"---------------------------------------------------------------
## Citation info
citation("HiCParser")

## ----"start", message=FALSE---------------------------------------------------
library("HiCParser")

## ----coolFormat---------------------------------------------------------------
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.cool",
    "path/to/condition-1.replicate-2.cool",
    "path/to/condition-1.replicate-3.cool",
    "path/to/condition-2.replicate-1.cool",
    "path/to/condition-2.replicate-2.cool",
    "path/to/condition-2.replicate-3.cool"
)

# For the sake of the example, we will use the same file, several times
paths <- rep(
    system.file("extdata",
        "hicsample_21.cool",
        package = "HiCParser"
    ),
    6
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# Instantiation of data set
hic.experiment <- parseCool(
    paths,
    conditions = conditions,
    replicates = replicates
)

## ----mcoolFormat--------------------------------------------------------------
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.mcool",
    "path/to/condition-1.replicate-2.mcool",
    "path/to/condition-1.replicate-3.mcool",
    "path/to/condition-2.replicate-1.mcool",
    "path/to/condition-2.replicate-2.mcool",
    "path/to/condition-2.replicate-3.mcool"
)

# For the sake of the example, we will use the same file, several times
paths <- rep(
    system.file("extdata",
        "hicsample_21.mcool",
        package = "HiCParser"
    ),
    6
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# mcool files can store several resolutions.
# We will mention the one we need.
binSize <- 5000000

# Instantiation of data set
# The same function "parseCool" is used for cool and mcool files
hic.experiment <- parseCool(
    paths,
    conditions = conditions,
    replicates = replicates,
    binSize = binSize # Specified for .mcool files.
)

## ----hicFormat----------------------------------------------------------------
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.hic",
    "path/to/condition-1.replicate-2.hic",
    "path/to/condition-2.replicate-1.hic",
    "path/to/condition-2.replicate-2.hic",
    "path/to/condition-3.replicate-1.hic"
)

# For the sake of the example, we will use the same file, several times
paths <- rep(
    system.file("extdata",
        "hicsample_21.hic",
        package = "HiCParser"
    ),
    6
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# hic files can store several resolutions.
# We will mention the one we need.
binSize <- 5000000

# Instantiation of data set
hic.experiment <- parseHiC(
    paths,
    conditions = conditions,
    replicates = replicates,
    binSize = binSize
)

## ----hicproFormat-------------------------------------------------------------
# Path to each matrix file
matrixPaths <- c(
    "path/to/condition-1.replicate-1.matrix",
    "path/to/condition-1.replicate-2.matrix",
    "path/to/condition-1.replicate-3.matrix",
    "path/to/condition-2.replicate-1.matrix",
    "path/to/condition-2.replicate-2.matrix",
    "path/to/condition-2.replicate-3.matrix"
)

# For the sake of the example, we will use the same file, several times
matrixPaths <- rep(
    system.file("extdata",
        "hicsample_21.matrix",
        package = "HiCParser"
    ),
    6
)

# Path to each bed file
bedPaths <- c(
    "path/to/condition-1.replicate-1.bed",
    "path/to/condition-1.replicate-2.bed",
    "path/to/condition-1.replicate-3.bed",
    "path/to/condition-2.replicate-1.bed",
    "path/to/condition-2.replicate-2.bed",
    "path/to/condition-2.replicate-3.bed"
)

# Alternatively, if the same bed file is used, we can provide it only once
bedPaths <- system.file("extdata",
    "hicsample_21.bed",
    package = "HiCParser"
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# Instantiation of data set
hic.experiment <- parseHiCPro(
    matrixPaths = matrixPaths,
    bedPaths = bedPaths,
    conditions = conditions,
    replicates = replicates
)

## ----tabFormat----------------------------------------------------------------
hic.experiment <- parseTabular(
    system.file("extdata",
        "hicsample_21.tsv",
        package = "HiCParser"
    ),
    sep = "\t"
)

## -----------------------------------------------------------------------------
library("HiCParser")
hicFilePath <- system.file("extdata", "hicsample_21.hic", package = "HiCParser")
hic.experiment <- parseHiC(
    paths = rep(hicFilePath, 6),
    binSize = 5000000,
    conditions = rep(seq(2), each = 3),
    replicates = rep(seq(3), 2)
)
hic.experiment

## -----------------------------------------------------------------------------
SummarizedExperiment::colData(hic.experiment)

## -----------------------------------------------------------------------------
head(SummarizedExperiment::assay(hic.experiment))

## -----------------------------------------------------------------------------
InteractionSet::interactions(hic.experiment)

## -----------------------------------------------------------------------------
path <- system.file("extdata", "hicsample_21.cool", package = "HiCParser")
object1 <- parseCool(path, conditions = 1, replicates = 1)
# Creating an object with a different condition
object2 <- parseCool(path, conditions = 2, replicates = 1)

## -----------------------------------------------------------------------------
objectMerged <- mergeInteractionSet(object1, object2)
SummarizedExperiment::colData(objectMerged)
head(SummarizedExperiment::assay(objectMerged))

## ----reproduce3, echo=FALSE-------------------------------------------------------------------------------------------
## Session info
library("sessioninfo")
options(width = 120)
session_info()

#' @title Merge two InteractionSet objects
#' @description
#' Merge two different \code{\link{InteractionSet}}.
#'
#' @param interactionSet1
#' The first \code{\link{InteractionSet}}.
#' @param interactionSet2
#' The second \code{\link{InteractionSet}}.
#' @param fill
#' Fill missing values with this.
#'
#' @return
#' The merged \code{\link{InteractionSet}}.
#'
#' @importFrom GenomicRanges union
#' @importFrom InteractionSet interactions
#' @importFrom BiocGenerics cbind
#' @examples
#' path <- system.file("extdata", "hicsample_21.cool", package = "HiCParser")
#' object1 <- parseCool(path, conditions = 1, replicates = 1)
#' # Creating an object with a different condition
#' object2 <- parseCool(path, conditions = 2, replicates = 1)
#' objectMerged <- mergeInteractionSet(object1, object2)
#'
#' @export
mergeInteractionSet <- function(interactionSet1, interactionSet2, fill = NA) {
    unionInteractions <- GenomicRanges::union(
        InteractionSet::interactions(interactionSet1),
        InteractionSet::interactions(interactionSet2)
    )
    # Complete InteractionSets
    interactionSet1 <- .fillInteractionSet(
        interactionSet1,
        unionInteractions,
        fill
    )
    interactionSet2 <- .fillInteractionSet(
        interactionSet2,
        unionInteractions,
        fill
    )

    # Merge
    newiset <- BiocGenerics::cbind(interactionSet1, interactionSet2)
    return(newiset)
}


#' Check input paths for parser functions (format and existence)
#'
#' @param ... a vector of paths, named, containing input data
#'
#' @return a  caracter vector of paths
#' @noRd
#'
#' @examples
#' path <-
#'     system.file("extdata", "hicsample_21.tsv", package = "HiCParser")
#' .checkPaths("ExamplePaths" = path)
.checkPaths <- function(...) {
    args <- list(...)
    path <- args[[1]]
    namePath <-
        names(vapply(match.call(), deparse, FUN.VALUE = "character"))[-1]
    if (is.factor(path)) {
        path <- as.vector(path)
    }
    if (anyNA(path)) {
        stop(namePath, " can't contain NA values.", call. = FALSE)
    }
    if (!is.character(path)) {
        stop(
            namePath, " must be a non empty vector of characters.",
            call. = FALSE
        )
    }
    for (p in path) {
        if (!file.exists(p)) {
            stop("'", p, "' does not exist.", call. = FALSE)
        }
    }
    return(path)
}

#' Check conditions and replicate arguments of parsers
#'
#' @param ... a vector of replicates or conditions
#'
#' @return a  caracter vector of paths
#' @noRd
#'
#' @examples
#' .checkConditionsReplicates(c(1, 1, 1, 2, 2, 2), c(1, 2, 3, 1, 2, 3))
.checkConditionsReplicates <- function(conditions, replicates) {
    if (is.factor(replicates)) {
        replicates <- as.vector(replicates)
    }
    if (is.factor(conditions)) {
        conditions <- as.vector(conditions)
    }

    if (is.null(replicates)) {
        stop("'replicates' must be a vector of replicates.", call. = FALSE)
    }
    if (is.null(conditions)) {
        stop("'conditions' must be a vector of conditions.", call. = FALSE)
    }
    if (anyNA(c(replicates, conditions))) {
        stop(
            "'replicates' and 'conditions' can't contain NA values",
            call. = FALSE
        )
    }
    if (length(conditions) != length(replicates)) {
        stop(
            "'conditions' and 'replicates' must have the same length",
            call. = FALSE
        )
    }
    return(list("conditions" = conditions, "replicates" = replicates))
}

test_that("mergeInteractionSet works as expected", {
    path <- system.file("extdata", "hicsample_21.tsv", package = "HiCParser")
    object1 <- parseTabular(path, sep = "\t")

    # Creating an 2nd fictive object, different from the first
    # Ginteractions
    set.seed(12345)
    allRegions <- regions(object1)
    gi2 <- InteractionSet::GInteractions(
        anchor1 = sample(allRegions$index, 15, replace = TRUE),
        anchor2 = sample(allRegions$index, 15, replace = TRUE),
        regions = allRegions,
        mode = "strict"
    )
    gi2 <- unique(gi2)

    # Reducing object1
    object1 <- object1[1:20]

    # Assays
    assays2 <- matrix(sample(seq_len(2000), length(gi2)), ncol = 1)
    # colData
    colData2 <- DataFrame("condition" = "2", "replicate" = "R2")
    # ISet object
    object2 <- InteractionSet::InteractionSet(assays2, gi2, colData = colData2)

    expect_error(
        objectMerged <- mergeInteractionSet(object1, object2),
        NA
    )
    expect_type(objectMerged, "S4")
    expect_true("InteractionSet" %in% class(objectMerged))
    expect_identical(
        objectMerged@colData,
        DataFrame(
            "condition" = c("1", "2"),
            "replicate" = c("R1", "R2")
        )
    )
    expect_true(length(objectMerged) == 28)
    nbNA <- apply(objectMerged@assays@data[[1]], 2, function(x) sum(is.na(x)))
    expect_identical(nbNA, c(8L, 14L))
    expect_true("StrictGInteractions" %in% class(objectMerged@interactions))
    expect_identical(
        which(objectMerged@interactions@anchor1 >
            objectMerged@interactions@anchor2),
        vector("integer", 0)
    )
})

test_that(".checkConditionsReplicates works as expected", {
    paths <- system.file("extdata", "hicsample_21.hic", package = "HiCParser")
    binSize <- 5000000
    expect_error(
        object <- parseHiC(rep(paths, 3),
            binSize = binSize,
            conditions = c(1, 2, 3),
            replicates = NULL
        ),
        "'replicates' must be a vector of replicates."
    )
    expect_error(
        object <- parseHiC(rep(paths, 3),
            binSize = binSize,
            conditions = NULL,
            replicates = c(1, 2, 3)
        ),
        "'conditions' must be a vector of conditions"
    )
    expect_error(
        object <- parseHiC(rep(paths, 3),
            binSize = binSize,
            conditions = c(1, NA, 3),
            replicates = c(1, 2, 3)
        ),
        "'replicates' and 'conditions' can't contain NA values"
    )
    expect_error(
        object <- parseHiC(rep(paths, 3),
            binSize = binSize,
            conditions = c(1, 2, 3),
            replicates = c(1, 3)
        ),
        "'conditions' and 'replicates' must have the same length"
    )

    expect_error(
        object <- parseHiC(rep(paths, 3),
            binSize = binSize,
            conditions = factor(c(1, 2, 3)),
            replicates = factor(c(1, 2, 3))
        ),
        NA
    )
    expect_identical(
        SummarizedExperiment::colData(object),
        S4Vectors::DataFrame(
            "condition" = c("1", "2", "3"),
            "replicate" = c("1", "2", "3")
        )
    )
})

test_that(".checkPaths works as expected", {
    paths <-
        system.file("extdata", "hicsample_21.hic", package = "HiCParser")
    binSize <- 5000000

    expect_error(
        object <- parseHiC(
            "paths" = c(paths, NA),
            binSize = binSize,
            conditions = c(1, 2),
            replicates = c(1, 2)
        ),
        "paths can't contain NA values."
    )
    expect_error(
        object <- parseHiC(
            "paths" = c(2, 1),
            binSize = binSize,
            conditions = c(1, 2),
            replicates = c(1, 2)
        ),
        "paths must be a non empty vector of characters."
    )
    expect_error(
        object <- parseHiC(
            "paths" = c(2, 1),
            binSize = binSize,
            conditions = c(1, 2),
            replicates = c(1, 2)
        ),
        "paths must be a non empty vector of characters."
    )
    expect_error(
        object <- parseHiC(
            "paths" = c("dont_exists"),
            binSize = binSize,
            conditions = c(1),
            replicates = c(1)
        ),
        "'dont_exists' does not exist."
    )
})

#' @title Initialize an InteractionSet object from tabular data
#' @description
#' Parses interactions in tabular format and fills the conditions, replicates,
#' and interactions slots of the provided \code{\link{InteractionSet}}.
#'
#' @param tabular
#' a path to a tabulated data file
#' @param input
#' The name of the input file
#' @param conditions
#' The names of the conditions
#' (infered from the table if not specified).
#' @param replicates
#' The names of the replicates
#' (infered from the table if not specified).
#'
#' @return
#' An \code{\link{InteractionSet}}.
#'
#' @keywords internal
#' @noRd
#' @importFrom data.table setorder setnames shift merge.data.table
#' @importFrom gtools mixedsort
#' @importFrom GenomicRanges GRanges
.setFromTabular <- function(tabular, conditions = NULL, replicates = NULL) {
    if (colnames(tabular)[1] != "chromosome") {
        stop(
            "First column of the input file must be named 'chromosome'.",
            call. = FALSE
        )
    }
    if (colnames(tabular)[2] == "position.1") {
        data.table::setnames(tabular, "position.1", "position 1")
    }
    if (colnames(tabular)[2] != "position 1") {
        stop(
            "Second column of the input file must be named 'position 1'.",
            call. = FALSE
        )
    }
    if (colnames(tabular)[3] == "position.2") {
        data.table::setnames(tabular, "position.2", "position 2")
    }
    if (colnames(tabular)[3] != "position 2") {
        stop(
            "Third column of the input file must be named 'position 2'.",
            call. = FALSE
        )
    }
    if (is.null(conditions) != is.null(replicates)) {
        stop(
            "Conditions and replicates should be both NULL, or none.",
            call. = FALSE
        )
    }
    tabular[, chromosome := as.character(chromosome)]
    tabular[, chromosome := factor(
        chromosome,
        levels = gtools::mixedsort(unique(chromosome))
    )]
    setorder(tabular, chromosome, `position 1`, `position 2`)
    # Assays part, fill with NA
    assays <- as.matrix(tabular[, 4:ncol(tabular), drop = FALSE])

    if (!is.null(conditions) | !is.null(replicates)) {
        if (
            (length(conditions) != ncol(assays)) |
                (length(replicates) != ncol(assays))
        ) {
            stop(
                "Number of conditions and replicates should match the number ",
                "of counts in the matrix.",
                call. = FALSE
            )
        }
    } else {
        if (!all(grepl("^.+?\\..+$", colnames(assays)))) {
            stop(
                "Fourth to last column of the input file must be named 'C.R', ",
                "with C the condition number/name and R the replicate ",
                "number/name.",
                call. = FALSE
            )
        }
    }
    assays[assays == 0] <- NA

    # GInteraction part
    tabular <- tabular[, .(chromosome, `position 1`, `position 2`)]
    data.table::setnames(tabular, "position 1", "bin.1")
    data.table::setnames(tabular, "position 2", "bin.2")

    diagonal <- (tabular$bin.1 == tabular$bin.2)
    binSize <- .modeVector(abs(
        tabular[!diagonal, ]$bin.1 - tabular[!diagonal, ]$bin.2
    ))

    tabular[, bin.1 := bin.1 / binSize]
    tabular[, bin.2 := bin.2 / binSize]

    allRegions <- data.table::melt(
        tabular[, .(chromosome, bin.1, bin.2)],
        id.vars = "chromosome",
        value.name = "indexC"
    )
    allRegions[, variable := NULL]
    allRegions <- unique(allRegions)
    data.table::setorder(allRegions, chromosome, indexC)

    # Constructing unique index for all chromosomes,
    # taking into account the difference in bins.
    allRegions[
        ,
        index := indexC - data.table::shift(indexC, fill = 0),
        by = .(chromosome)
    ]
    allRegions[index == 0, index := 1]
    allRegions[, index := cumsum(index)]
    allRegions[, end := (indexC + 1) * binSize]
    allRegions[, start := (indexC) * binSize + 1]
    data.table::setcolorder(
        allRegions,
        c("chromosome", "start", "end", "index", "indexC")
    )

    tabular <- data.table::merge.data.table(
        tabular,
        allRegions[, .(chromosome, startIndex = index, bin.1 = indexC)],
        all.x = TRUE,
        sort = FALSE,
        by = c("chromosome", "bin.1")
    )
    tabular <- data.table::merge.data.table(
        tabular,
        allRegions[, .(chromosome, stopIndex = index, bin.2 = indexC)],
        all.x = TRUE,
        sort = FALSE,
        by = c("chromosome", "bin.2")
    )

    allRegions[, indexC := NULL]
    order1 <- match(tabular$startIndex, allRegions$index)
    order2 <- match(tabular$stopIndex, allRegions$index)
    allRegions <- GenomicRanges::GRanges(allRegions)

    gi <- InteractionSet::GInteractions(
        allRegions[order1],
        allRegions[order2],
        regions = allRegions,
        mode = "strict"
    )

    if (is.null(conditions)) {
        conditions <- gsub("^(.+?)\\..+$", "\\1", colnames(assays))
    }
    if (is.null(replicates)) {
        replicates <- gsub("^.+?\\.(.+)$", "\\1", colnames(assays))
    }

    interactionSet <-
        .createInteractionSet(assays, gi, allRegions, conditions, replicates)
    return(interactionSet)
}

#' @title Parser for tabular data
#' @description
#' Read the file, produce an \code{InteractionSet} object.
#'
#' @param path
#' A path to a tabular file.
#' @param sep
#' The separator of the tabular file. Default to tabulation.
#'
#' @details
#' Accepts a tabular file with \code{chromosome}, \code{position 1},
#' \code{position 2}, and multiple replicate columns listing interaction counts.
#' Null interactions do not have to be listed. Values must be separated by
#' tabulations. The header must be
#' \code{chromosome position 1 position 2 x.y x.y x.y ...} with \code{x}
#' replaced by condition names and \code{y} replaced by replicate names.
#'
#' @return
#' An InteractionSet object.
#'
#' @examples
#' path <- system.file("extdata", "hicsample_21.tsv", package = "HiCParser")
#' object <- parseTabular(path, sep = "\t")
#'
#' @usage
#' parseTabular(path, sep = '\t')
#'
#' @importFrom data.table fread
#' @export
parseTabular <- function(path, sep = "\t") {
    path <- .checkPaths("path" = path)
    if (length(path) > 1) {
        stop("'path' must be 1-lengths character vector.", call. = FALSE)
    }

    message("Parsing '", path, "'.")

    interactions <- data.table::fread(
        file = path,
        sep = sep,
        header = TRUE,
        # comment.char = "#",
        check.names = FALSE,
        data.table = TRUE,
        stringsAsFactors = FALSE
    )

    interactionSet <- .setFromTabular(interactions)
    interactionSet <- .sortHiCData(interactionSet)

    return(interactionSet)
}

test_that("tabular parser for one file produce correct format", {
    path <- system.file("extdata", "hicsample_21.tsv", package = "HiCParser")
    expect_message(object <- parseTabular(path), "hicsample_21.tsv")
    # Class and slots
    expect_s4_class(object, "InteractionSet")

    # Here test order of interactions and assays.
    expect_equal(length(object), 44)

    # Interactions
    expect_true("matrix" %in% class(SummarizedExperiment::assay(object)))
    expect_s4_class(InteractionSet::regions(object), "GRanges")
    expect_s4_class(InteractionSet::interactions(object), "StrictGInteractions")
    expect_s4_class(S4Vectors::mcols(object), "DataFrame")
    expect_true(is.numeric(SummarizedExperiment::assay(object)))
})

#' @description
#' Parses a single interactions file in \code{.hic} format. Calls the C++
#' \code{parseHiCFile} parser.
#'
#' @param path
#' The path to the interactions file.
#' @param binSize
#' The resolution (span of each position in number of bases) to select within
#' the \code{.hic} file.
#'
#' @return
#' An \code{\link{InteractionSet}}.
#'
#' @keywords internal
#' @noRd
#' @importFrom data.table setalloccol
.parseOneHiC <- function(path, binSize, condition, replicate) {
    message("\nParsing '", path, "'.")
    interactions <- parseHiCFile(path, binSize)
    # Automagical stuff to transform Rcpp DataFrame to data.table
    interactions <- data.table::setalloccol(interactions)
    # Create InteractionSet object
    interactions <- .setFromTabular(interactions, condition, replicate)
    return(interactions)
}

#' @title Parser for data in hic format
#' @description
#' Parses interactions in \code{.hic} format and returns an InteractionSet
#' object.
#'
#' @param paths
#' A vector of paths to \code{.hic} files.
#' @param binSize
#' The resolution (span of each position in number of bases) to select within
#' the \code{.hic} files.
#' @param conditions
#' A vector of condition names repeated along the replicates.
#' @param replicates
#' A vector of replicate names repeated along the conditions.
#' @return
#' An InteractionSet.
#'
#' @examples
#' # Path to each file
#' paths <- c(
#'     "path/to/condition-1.replicate-1.hic",
#'     "path/to/condition-1.replicate-2.hic",
#'     "path/to/condition-1.replicate-3.hic",
#'     "path/to/condition-2.replicate-1.hic",
#'     "path/to/condition-2.replicate-2.hic",
#'     "path/to/condition-2.replicate-3.hic"
#' )
#' # Replicate and condition of each file. Can be names instead of numbers.
#' conditions <- c(1, 1, 1, 2, 2, 2)
#' replicates <- c(1, 2, 3, 1, 2, 3)
#' # Resolution to select
#' binSize <- 500000
#' if (FALSE) {
#'     # Instantiation of data set
#'     hic.experiment <- parseHiC(
#'         paths,
#'         conditions = conditions,
#'         replicates = replicates,
#'         binSize = binSize
#'     )
#' }
#' @usage
#' parseHiC(paths, binSize, conditions, replicates)
#'
#' @importFrom pbapply pbmapply
#' @export
parseHiC <- function(paths, binSize, conditions, replicates) {
    paths <- .checkPaths("paths" = paths)
    repCond <- .checkConditionsReplicates(conditions, replicates)
    if (min(lengths(repCond)) != length(paths)) {
        stop(
            "'conditions/replicates' and 'paths' ",
            "must have the same length",
            call. = FALSE
        )
    }

    if (!is.numeric(binSize) || length(binSize) != 1) {
        stop("'binSize' must be an integer.", call. = FALSE)
    }
    binSize <- as.integer(binSize)

    interactionSet <- pbapply::pbmapply(
        .parseOneHiC,
        path      = paths,
        binSize   = binSize,
        condition = repCond[["conditions"]],
        replicate = repCond[["replicates"]]
    )

    mergedinteractionSet <- Reduce(
        f = mergeInteractionSet,
        x = interactionSet
    )
    mergedinteractionSet <- .sortHiCData(mergedinteractionSet)

    return(mergedinteractionSet)
}

test_that("Parser for .hic data works as expected", {
    paths <-
        system.file("extdata", "hicsample_21.hic", package = "HiCParser")

    # Replicate and condition of each file. Can be names instead of numbers.
    replicates <- 1
    conditions <- 1
    binSize <- 5000000

    # Instantiation of data set
    expect_message(
        object <- parseHiC(
            paths,
            binSize = binSize,
            replicates = replicates,
            conditions = conditions
        ),
        "hicsample_21.hic"
    )
    expect_equal(length(object), 44)

    # Interactions
    expect_true("matrix" %in% class(SummarizedExperiment::assay(object)))
    expect_s4_class(InteractionSet::regions(object), "GRanges")
    expect_s4_class(InteractionSet::interactions(object), "StrictGInteractions")
    expect_s4_class(S4Vectors::mcols(object), "DataFrame")
    expect_true(is.numeric(SummarizedExperiment::assay(object)))
})

#' .modeVector Extract the mode of vector.
#'
#' @param x A vector
#'
#' @return The mode of a vector
#'
#' @examples
#' .modeVector(c(1, 2, 2, 2, 4))
#' @noRd
.modeVector <- function(x) {
    ux <- unique(x)
    ux[which.max(tabulate(match(x, ux)))]
}

#' @description
#' Fills parameters and slots describing the data. Called by a
#' \code{\link{HiCParser}} constructor.
#'
#' @param object
#' A \code{InteractionSet}.
#'
#' @return
#' A sorted \code{InteractionSet}.
#'
#' @keywords internal
#' @importFrom Seqinfo seqnames seqlevels
#' @importFrom InteractionSet reduceRegions regions anchors
#' @importFrom gtools mixedsort
#' @importFrom S4Vectors Rle DataFrame
#' @noRd
.sortHiCData <- function(object) {
    # Reduce the levels in interaction part
    object <- InteractionSet::reduceRegions(object)
    objectRegions <- InteractionSet::regions(object)
    chromosomeNames <- unique(as.character(
        Seqinfo::seqnames(objectRegions)
    ))
    chromosomeNames <- gtools::mixedsort(chromosomeNames)
    Seqinfo::seqlevels(
        InteractionSet::regions(object),
        pruning.mode = "coarse"
    ) <- chromosomeNames

    # Add chromosome column for split purpose
    chromosomes <-
        Seqinfo::seqnames(InteractionSet::anchors(object, "first"))
    chromosomes <- S4Vectors::Rle(factor(chromosomes, levels = chromosomeNames))
    S4Vectors::mcols(object) <- S4Vectors::DataFrame("chromosome" = chromosomes)

    # Sorting interactions and assay
    ids <- InteractionSet::anchors(object, id = TRUE)
    neworder <- order(chromosomes, ids$first, ids$second)
    object <- object[neworder, ]
    return(object)
}

#' Create the object InteractionSet from given conditions and replicates
#'
#' @param assay matrix of assays
#' @param gi GInteractions object
#' @param allRegions regions object
#' @param condition condition (for colData), vector of length 1 or more
#' @param replicate replicate (for colData), vector of length 1 or more
#'
#' @return an interactionSet object
#' @keywords internal
#' @noRd
#' @importFrom S4Vectors DataFrame
#' @importFrom InteractionSet anchors GInteractions InteractionSet intrachr
.createInteractionSet <- function(assay, gi, allRegions, condition, replicate) {
    # Add 0 on the diagonal if there is only off diagonal interaction
    ids <- InteractionSet::anchors(gi, id = TRUE)
    idsDiagonals <- ids$first[ids$first == ids$second]
    notPresent <- setdiff(unique(c(ids$first, ids$second)), idsDiagonals)
    nb <- length(notPresent)
    colnames(assay) <- NULL
    if (nb > 0) {
        notPresentRegions <- allRegions[match(notPresent, allRegions$index)]
        gi <- c(
            gi,
            InteractionSet::GInteractions(
                notPresentRegions,
                notPresentRegions,
                regions = allRegions,
                mode = "strict"
            )
        )
        assay <- rbind(assay, as.matrix(rep(0, nb * ncol(assay)),
            ncol = ncol(assay),
            nrow = nb
        ))
    }

    interactionSet <- InteractionSet::InteractionSet(
        assays = assay,
        interactions = gi,
        colData = S4Vectors::DataFrame(
            "condition" = condition,
            "replicate" = replicate
        )
    )

    # Keep only intra-chromosomal interactions
    interactionSet <- interactionSet[InteractionSet::intrachr(interactionSet), ]
    return(interactionSet)
}

#' @description
#' Fill \code{\link{InteractionSet}} with possibly missing values
#'
#' @param interactionSet
#' An \code{\link{InteractionSet}}.
#' @param interactionSetUnion
#' The full \code{\link{InteractionSet}}.
#' @param fill
#' Fill missing values with this.
#'
#' @return
#' The full \code{\link{InteractionSet}}.
#'
#' @keywords internal
#' @noRd
#' @importFrom InteractionSet InteractionSet
#' @importFrom GenomicRanges match
#' @importFrom SummarizedExperiment assay colData
.fillInteractionSet <- function(
    interactionSet,
    interactionSetUnion,
    fill = NA) {
    over <- GenomicRanges::match(interactionSet, interactionSetUnion)
    totalColumns <- ncol(interactionSet)
    newAssays <- matrix(
        rep(fill, length(interactionSetUnion) * totalColumns),
        ncol = totalColumns
    )
    newAssays[over, ] <- SummarizedExperiment::assay(interactionSet)
    return(
        InteractionSet::InteractionSet(
            newAssays,
            interactionSetUnion,
            colData = SummarizedExperiment::colData(interactionSet)
        )
    )
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

parseHiCFile <- function(fname, resolution) {
    .Call('_HiCParser_parseHiCFile', PACKAGE = 'HiCParser', fname, resolution)
}

#' @description
#' Parses a single pair of \code{.matrix} and \code{.bed} files.
#'
#' @param matrixPath
#' The path to the interactions matrix file.
#' @param bedPath
#' The path to the bed file.
#'
#' @return
#' A data.table of interactions.
#'
#' @keywords internal
#' @noRd
#' @importFrom InteractionSet GInteractions
#' @importFrom GenomicRanges GRanges
#' @importFrom data.table setorder
.parseOneHiCPro <- function(matrixPath, bedPath, condition, replicate) {
    message("\nParsing '", matrixPath, "' and '", bedPath, "'.")

    interactions <- data.table::fread(
        matrixPath,
        header = FALSE,
        stringsAsFactors = FALSE,
        col.names = c("startIndex", "stopIndex", "interaction"),
        data.table = TRUE
    )

    bed <- data.table::fread(
        bedPath,
        header = FALSE,
        stringsAsFactors = FALSE,
        col.names = c("chromosome", "start", "end", "index"),
        data.table = TRUE
    )

    setorder(bed, chromosome, start, end)
    # Adding 1 to follow Bioconductor GRanges recommended format
    bed[, start := start + 1]
    # Keeping only intra-chromosomal interactions
    # Add 1 if BED index start with 0
    allChromosomes <- vector("character", length = max(bed$index) + 1)
    allChromosomes[bed[, index] + 1] <- bed[, chromosome]
    interactions <- interactions[
        allChromosomes[startIndex + 1] == allChromosomes[stopIndex + 1]
    ]

    order1 <- match(interactions$startIndex, bed$index)
    order2 <- match(interactions$stopIndex, bed$index)
    allRegions <- GenomicRanges::GRanges(bed)

    gi <- InteractionSet::GInteractions(
        allRegions[order1],
        allRegions[order2],
        regions = allRegions,
        mode = "strict"
    )
    assay <- as.matrix(interactions$interaction, ncol = 1)
    interactionSet <-
        .createInteractionSet(assay, gi, allRegions, condition, replicate)

    return(interactionSet)
}

#' @title Parser for HiCPro data
#' @description
#' Parses interactions in pairs of \code{.matrix} and \code{.bed} files and
#' returns an InteractionSet object.
#'
#' @param matrixPaths
#' A vector of paths to HiC-Pro matrix files.
#' @param bedPaths
#' A vector of paths to HiC-Pro bed files.
#' @param conditions
#' A vector of condition names repeated along the replicates.
#' @param replicates
#' A vector of replicate names repeated along the conditions.
#'
#' @return
#' An InteractionSet.
#'
#' @examples
#' # Path to each matrix file
#' matrixPaths <- c(
#'     "path/to/condition-1.replicate-1.matrix",
#'     "path/to/condition-1.replicate-2.matrix",
#'     "path/to/condition-1.replicate-3.matrix",
#'     "path/to/condition-2.replicate-1.matrix",
#'     "path/to/condition-2.replicate-2.matrix",
#'     "path/to/condition-2.replicate-3.matrix"
#' )
#'
#' # Path to each bed file
#' bedPaths <- c(
#'     "path/to/condition-1.replicate-1.bed",
#'     "path/to/condition-1.replicate-2.bed",
#'     "path/to/condition-1.replicate-3.bed",
#'     "path/to/condition-2.replicate-1.bed",
#'     "path/to/condition-2.replicate-2.bed",
#'     "path/to/condition-2.replicate-3.bed"
#' )
#'
#' # Condition and replicate of each file. Can be names instead of numbers.
#' conditions <- c(1, 1, 1, 2, 2, 2)
#' replicates <- c(1, 2, 3, 1, 2, 3)
#'
#' if (FALSE) {
#'     # Instantiation of data set
#'     hic.experiment <- parseHiCPro(
#'         matrixPaths = matrixPaths,
#'         bedPaths = bedPaths,
#'         conditions = conditions,
#'         replicates = replicates
#'     )
#' }
#'
#' @usage
#' parseHiCPro(matrixPaths, bedPaths, conditions, replicates)
#'
#' @importFrom pbapply pbmapply
#' @export
parseHiCPro <- function(matrixPaths, bedPaths, conditions, replicates) {
    matrixPaths <- .checkPaths("matrixPaths" = matrixPaths)
    bedPaths <- .checkPaths("bedPaths" = bedPaths)

    if (length(bedPaths) == 1) {
        bedPaths <- rep(bedPaths, length(matrixPaths))
    }
    if (length(matrixPaths) != length(bedPaths)) {
        stop(
            "'matrixPaths' and 'bedPaths' must have the same length.",
            call. = FALSE
        )
    }

    repCond <- .checkConditionsReplicates(conditions, replicates)
    if (min(lengths(repCond)) != length(matrixPaths)) {
        stop(
            "'conditions/replicates' and 'matrixPaths/bedPaths' ",
            "must have the same length",
            call. = FALSE
        )
    }

    interactionSet <- pbapply::pbmapply(
        .parseOneHiCPro,
        matrixPaths,
        bedPaths,
        repCond[["conditions"]],
        repCond[["replicates"]]
    )

    mergedinteractionSet <- Reduce(f = mergeInteractionSet, x = interactionSet)
    mergedinteractionSet <- .sortHiCData(mergedinteractionSet)

    return(mergedinteractionSet)
}

#' @description
#' Parses a single interactions file in \code{.cool} or \code{.mcool} format.
#'
#' @param path
#' The path to the interactions file.
#' @param binSize
#' The resolution (span of each position in number of bases). Optionally
#' provided to select the appropriate resolution in \code{.mcool} files.
#' Defaults to NULL.
#' @details
#' To read `.cool` of `.mcool` files, the `rhdf5` package is required.
#' Please install it before running the function.
#'
#' @return
#' A data.table of interactions.
#'
#' @keywords internal
#' @noRd
#' @importFrom data.table setorder
#' @importFrom GenomicRanges GRanges
.parseOneCool <- function(path, binSize = NA, condition, replicate) {
    if (!requireNamespace("rhdf5")) {
        stop("'rhdf5' package is required. Please install it and retry.")
    }
    message("\nParsing '", path, "'.")
    uri <- function(path) {
        if (!is.numeric(binSize)) {
            return(path)
        }
        return(
            paste(
                "resolutions",
                format(binSize, scientific = FALSE),
                path,
                sep = "/"
            )
        )
    }

    bins <- data.table::data.table(
        chromosome = factor(
            rhdf5::h5read(file = path, name = uri("bins/chrom"))
        ),
        start = rhdf5::h5read(file = path, name = uri("bins/start")),
        end = rhdf5::h5read(file = path, name = uri("bins/end"))
    )
    bins[, start := as.integer(start)]
    bins[, start := start + 1]
    bins[, end := as.integer(end)]

    data.table::setorder(bins, chromosome, start, end)
    bins[, index := seq_len(nrow(bins))]

    interactions <- data.table::data.table(
        id1 = rhdf5::h5read(file = path, name = uri("pixels/bin1_id")),
        id2 = rhdf5::h5read(file = path, name = uri("pixels/bin2_id")),
        interaction = rhdf5::h5read(file = path, name = uri("pixels/count"))
    )
    interactions[, id1 := as.integer(id1) + 1]
    interactions[, id2 := as.integer(id2) + 1]
    interactions[, interaction := as.numeric(interaction)]

    order1 <- match(interactions$id1, bins$index)
    order2 <- match(interactions$id2, bins$index)
    allRegions <- GenomicRanges::GRanges(bins)

    # GInteractions part
    gi <- InteractionSet::GInteractions(
        allRegions[order1],
        allRegions[order2],
        regions = allRegions,
        mode = "strict"
    )
    assay <- as.matrix(interactions$interaction, ncol = 1)

    interactionSet <-
        .createInteractionSet(assay, gi, allRegions, condition, replicate)
    return(interactionSet)
}

#' @title Parser for data in cool format
#' @description
#' Parses interactions in \code{.cool} or \code{.mcool} format and returns
#' an InteractionSet object.
#' @param paths
#' A vector of paths to \code{.cool} or \code{.mcool} files.
#' @param conditions
#' A vector of condition names repeated along the replicates.
#' @param replicates
#' A vector of replicate names repeated along the conditions.
#' @param binSize
#' The resolution (span of each position in number of bases). Optionally
#' provided to select the appropriate resolution in \code{.mcool} files.
#' Defaults to NULL.
#' @details
#' To read `.cool` of `.mcool` files, the `rhdf5` package is required.
#' Please install it before running the function.
#'
#' @return
#' An InteractionSet.
#' @examples
#' # EXAMPLE FOR .cool FORMAT
#' # Path to each file
#' pathsCool <- c(
#'     "path/to/condition-1.replicate-1.cool",
#'     "path/to/condition-1.replicate-2.cool",
#'     "path/to/condition-1.replicate-3.cool",
#'     "path/to/condition-2.replicate-1.cool",
#'     "path/to/condition-2.replicate-2.cool",
#'     "path/to/condition-2.replicate-3.cool"
#' )
#' # Condition and replicate of each file. Can be names instead of numbers.
#' conditions <- c(1, 1, 1, 2, 2, 2)
#' replicates <- c(1, 2, 3, 1, 2, 3)
#' if (FALSE) {
#'     library(rhdf5)
#'     object <- parseCool(
#'         paths,
#'         conditions = conditions,
#'         replicates = replicates
#'     )
#' }
#'
#' # EXAMPLE FOR .mcool FORMAT
#' # Resolution
#' binSize <- 500000
#' # Path to each file
#' paths <- c(
#'     "path/to/condition-1.replicate-1.mcool",
#'     "path/to/condition-1.replicate-2.mcool",
#'     "path/to/condition-1.replicate-3.mcool",
#'     "path/to/condition-2.replicate-1.mcool",
#'     "path/to/condition-2.replicate-2.mcool",
#'     "path/to/condition-2.replicate-3.mcool"
#' )
#' # Condition and replicate of each file. Can be names instead of numbers.
#' conditions <- c(1, 1, 1, 2, 2, 2)
#' replicates <- c(1, 2, 3, 1, 2, 3)
#' if (FALSE) {
#'     # Instantiation of data set
#'     library(rhdf5)
#'     object <- parseCool(
#'         paths,
#'         conditions = conditions,
#'         replicates = replicates,
#'         binSize = binSize
#'     )
#' }
#' @importFrom pbapply pbmapply
#' @export
parseCool <- function(paths, binSize = NA, conditions, replicates) {
    if (!requireNamespace("rhdf5")) {
        stop("'rhdf5' package is required. Please install it and retry.")
    }
    paths <- .checkPaths("paths" = paths)

    repCond <- .checkConditionsReplicates(conditions, replicates)
    if (min(lengths(repCond)) != length(paths)) {
        stop(
            "'conditions/replicates' and 'paths' must have the same length",
            call. = FALSE
        )
    }

    if (!is.na(binSize) && all(grepl(".cool", paths, fixed = TRUE))) {
        warning("binSize specified but all files are not .cool, ignored")
        binSize <- NA
    }
    if (is.na(binSize) && any(grepl("mcool", paths, fixed = TRUE))) {
        stop("binSize must be specified fot mcool files")
    }
    if (!is.na(binSize) && (!is.numeric(binSize) || length(binSize) != 1)) {
        stop("'binSize' must be an integer.", call. = FALSE)
    }

    interactionSetCool <- pbapply::pbmapply(
        .parseOneCool,
        path = paths,
        binSize = binSize,
        condition = repCond[["conditions"]],
        replicate = repCond[["replicates"]]
    )

    mergedinteractionSetCool <- Reduce(mergeInteractionSet, interactionSetCool)
    mergedinteractionSetCool <- .sortHiCData(mergedinteractionSetCool)

    return(mergedinteractionSetCool)
}

test_that("Parser for .cool data works as expected", {
    paths <-
        system.file("extdata", "hicsample_21.cool",
            package = "HiCParser"
        )

    # Replicate and condition of each file. Can be names instead of numbers.
    replicates <- "GG"
    conditions <- "AA"
    binSize <- 5000000

    # Instantiation of data set
    expect_message(
        object <- parseCool(
            paths,
            replicates = replicates,
            conditions = conditions
        ),
        "hicsample_21.cool"
    )
    expect_equal(length(object), 44)

    # Interactions
    expect_true("matrix" %in% class(SummarizedExperiment::assay(object)))
    expect_s4_class(InteractionSet::regions(object), "GRanges")
    expect_s4_class(
        InteractionSet::interactions(object),
        "StrictGInteractions"
    )
    expect_s4_class(S4Vectors::mcols(object), "DataFrame")
    expect_true(is.numeric(SummarizedExperiment::assay(object)))
})

test_that("Parser for .cool data returns expected errors", {
    paths <-
        system.file("extdata", "hicsample_21.cool",
            package = "HiCParser"
        )
    expect_error(
        parseCool(
            paths,
            replicates = c(1, 2),
            conditions = c(1, 2)
        ),
        "'conditions/replicates' and 'paths' must have the same length"
    )
})

---
title: "Introduction to HiCParser"
author: 
  - name: Elise Maigné
    affiliation:
    - INRAE, MIAT
    email: elise.maigne@inrae.fr
  - name: Matthias Zytnicki
    affiliation:
    - INRAE, MIAT
    email: matthias.zytnicki@inrae.fr
output: 
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    code_folding: show
date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('HiCParser')`"
vignette: >
  %\VignetteIndexEntry{Introduction to HiCParser}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL,
    warning = FALSE
)
```

# Basics

## Required knowledge

`r BiocStyle::Biocpkg("HiCParser")` is based on other packages and in 
particular in those that have implemented the infrastructure needed for 
dealing with HiC data with several replicates and conditions. Is provides 
several parsers, for several HiC data standard format to import them 
into R in a `r BiocStyle::Biocpkg("InteractionSet")` object.

## Citing `HiCParser`

We hope that `r BiocStyle::Biocpkg("HiCParser")` will be useful for your 
research. 
Please use the following information to cite the package and the overall 
approach. Thank you!

```{r "citation"}
## Citation info
citation("HiCParser")
```

# Start using `HiCParser`

```{r "start", message=FALSE}
library("HiCParser")
```

`HiCParser` can import Hi-C data sets in various different formats:
- Cooler `.cool` or `.mcool` files.
- Juicer `.hic` files.
- HiC-Pro `.matrix` and `.bed` files.
- Tabular (`.tsv`, `.csv`, ...) files.

## Cooler files

### `.cool` files

To load `.cool` files generated by [Cooler][cooler-documentation]
[@cooler]:

```{r coolFormat}
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.cool",
    "path/to/condition-1.replicate-2.cool",
    "path/to/condition-1.replicate-3.cool",
    "path/to/condition-2.replicate-1.cool",
    "path/to/condition-2.replicate-2.cool",
    "path/to/condition-2.replicate-3.cool"
)

# For the sake of the example, we will use the same file, several times
paths <- rep(
    system.file("extdata",
        "hicsample_21.cool",
        package = "HiCParser"
    ),
    6
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# Instantiation of data set
hic.experiment <- parseCool(
    paths,
    conditions = conditions,
    replicates = replicates
)
```


### `.mcool` files

To load `.mcool` files generated by [Cooler][cooler-documentation]
[@cooler]:

```{r mcoolFormat}
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.mcool",
    "path/to/condition-1.replicate-2.mcool",
    "path/to/condition-1.replicate-3.mcool",
    "path/to/condition-2.replicate-1.mcool",
    "path/to/condition-2.replicate-2.mcool",
    "path/to/condition-2.replicate-3.mcool"
)

# For the sake of the example, we will use the same file, several times
paths <- rep(
    system.file("extdata",
        "hicsample_21.mcool",
        package = "HiCParser"
    ),
    6
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# mcool files can store several resolutions.
# We will mention the one we need.
binSize <- 5000000

# Instantiation of data set
# The same function "parseCool" is used for cool and mcool files
hic.experiment <- parseCool(
    paths,
    conditions = conditions,
    replicates = replicates,
    binSize = binSize # Specified for .mcool files.
)
```

## hic files

To load `.hic` files generated by [Juicer][juicer-documentation] [@juicer]:

```{r hicFormat}
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.hic",
    "path/to/condition-1.replicate-2.hic",
    "path/to/condition-2.replicate-1.hic",
    "path/to/condition-2.replicate-2.hic",
    "path/to/condition-3.replicate-1.hic"
)

# For the sake of the example, we will use the same file, several times
paths <- rep(
    system.file("extdata",
        "hicsample_21.hic",
        package = "HiCParser"
    ),
    6
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# hic files can store several resolutions.
# We will mention the one we need.
binSize <- 5000000

# Instantiation of data set
hic.experiment <- parseHiC(
    paths,
    conditions = conditions,
    replicates = replicates,
    binSize = binSize
)
```
Currently, `HiCParser` supports the hic format up to the version 9.

## HiC-Pro files

To load `.matrix` and `.bed` files generated by [HiC-Pro][hicpro-documentation]
[@hicpro]:

```{r hicproFormat}
# Path to each matrix file
matrixPaths <- c(
    "path/to/condition-1.replicate-1.matrix",
    "path/to/condition-1.replicate-2.matrix",
    "path/to/condition-1.replicate-3.matrix",
    "path/to/condition-2.replicate-1.matrix",
    "path/to/condition-2.replicate-2.matrix",
    "path/to/condition-2.replicate-3.matrix"
)

# For the sake of the example, we will use the same file, several times
matrixPaths <- rep(
    system.file("extdata",
        "hicsample_21.matrix",
        package = "HiCParser"
    ),
    6
)

# Path to each bed file
bedPaths <- c(
    "path/to/condition-1.replicate-1.bed",
    "path/to/condition-1.replicate-2.bed",
    "path/to/condition-1.replicate-3.bed",
    "path/to/condition-2.replicate-1.bed",
    "path/to/condition-2.replicate-2.bed",
    "path/to/condition-2.replicate-3.bed"
)

# Alternatively, if the same bed file is used, we can provide it only once
bedPaths <- system.file("extdata",
    "hicsample_21.bed",
    package = "HiCParser"
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

# Instantiation of data set
hic.experiment <- parseHiCPro(
    matrixPaths = matrixPaths,
    bedPaths = bedPaths,
    conditions = conditions,
    replicates = replicates
)
```

## Tabular files

A tabular file is a tab-separated multi-replicate sparse matrix with a header:

```
chromosome    position 1    position 2    C1.R1    C1.R2    C1.R3    ...
Y             1500000       7500000       145      184      72       ...
```

The number of interactions between `position 1` and `position 2` of
`chromosome` are reported in each `condition.replicate` column. There is no
limit to the number of conditions and replicates.

To load Hi-C data in this format:

```{r tabFormat}
hic.experiment <- parseTabular(
    system.file("extdata",
        "hicsample_21.tsv",
        package = "HiCParser"
    ),
    sep = "\t"
)
```

# InteractionSet format


# Output : InteractionSet format

The output is a `r BiocStyle::Biocpkg("InteractionSet")`.
This object can store one or several samples.

Please read the documentation associated with the 
`r BiocStyle::Biocpkg("InteractionSet")` package to known more about this 
format.

```{r}
library("HiCParser")
hicFilePath <- system.file("extdata", "hicsample_21.hic", package = "HiCParser")
hic.experiment <- parseHiC(
    paths = rep(hicFilePath, 6),
    binSize = 5000000,
    conditions = rep(seq(2), each = 3),
    replicates = rep(seq(3), 2)
)
hic.experiment
```

The conditions and replicates are reported in the `colData` slot : 

```{r}
SummarizedExperiment::colData(hic.experiment)
```

They corresponds to columns of the `assays` matrix (containing 
interactions values):

```{r}
head(SummarizedExperiment::assay(hic.experiment))
```

The positions of interactions are in the `interactions` slot of the object:

```{r}
InteractionSet::interactions(hic.experiment)
```

## Additional utils functions

A function `mergeInteractionSet` to merge `InteractionSet` objects, 
from the same experiment (for differents replicates or conditions).

It merges the the data containing bins of interactions and fill the assays 
matrix accordingly, returning an assays matrix with several columns.

The object returned by the function is an `InteractionSet`.

Here is a fictitious example:

```{r}
path <- system.file("extdata", "hicsample_21.cool", package = "HiCParser")
object1 <- parseCool(path, conditions = 1, replicates = 1)
# Creating an object with a different condition
object2 <- parseCool(path, conditions = 2, replicates = 1)
```

The merged object:

```{r}
objectMerged <- mergeInteractionSet(object1, object2)
SummarizedExperiment::colData(objectMerged)
head(SummarizedExperiment::assay(objectMerged))
```

# Reproducibility

This package was developed using `r BiocStyle::Biocpkg("biocthis")`.

`R` session information.

```{r reproduce3, echo=FALSE}
## Session info
library("sessioninfo")
options(width = 120)
session_info()
```


# Bibliography

Lun ATL, Perry M and Ing-Simmons E (2016). 
Infrastructure for genomic interactions: Bioconductor classes for Hi-C, 
ChIA-PET and related experiments. F1000Res. 5, 950

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hicproParser.R
\name{parseHiCPro}
\alias{parseHiCPro}
\title{Parser for HiCPro data}
\usage{
parseHiCPro(matrixPaths, bedPaths, conditions, replicates)
}
\arguments{
\item{matrixPaths}{A vector of paths to HiC-Pro matrix files.}

\item{bedPaths}{A vector of paths to HiC-Pro bed files.}

\item{conditions}{A vector of condition names repeated along the replicates.}

\item{replicates}{A vector of replicate names repeated along the conditions.}
}
\value{
An InteractionSet.
}
\description{
Parses interactions in pairs of \code{.matrix} and \code{.bed} files and
returns an InteractionSet object.
}
\examples{
# Path to each matrix file
matrixPaths <- c(
    "path/to/condition-1.replicate-1.matrix",
    "path/to/condition-1.replicate-2.matrix",
    "path/to/condition-1.replicate-3.matrix",
    "path/to/condition-2.replicate-1.matrix",
    "path/to/condition-2.replicate-2.matrix",
    "path/to/condition-2.replicate-3.matrix"
)

# Path to each bed file
bedPaths <- c(
    "path/to/condition-1.replicate-1.bed",
    "path/to/condition-1.replicate-2.bed",
    "path/to/condition-1.replicate-3.bed",
    "path/to/condition-2.replicate-1.bed",
    "path/to/condition-2.replicate-2.bed",
    "path/to/condition-2.replicate-3.bed"
)

# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)

if (FALSE) {
    # Instantiation of data set
    hic.experiment <- parseHiCPro(
        matrixPaths = matrixPaths,
        bedPaths = bedPaths,
        conditions = conditions,
        replicates = replicates
    )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hicParser.R
\name{parseHiC}
\alias{parseHiC}
\title{Parser for data in hic format}
\usage{
parseHiC(paths, binSize, conditions, replicates)
}
\arguments{
\item{paths}{A vector of paths to \code{.hic} files.}

\item{binSize}{The resolution (span of each position in number of bases) to select within
the \code{.hic} files.}

\item{conditions}{A vector of condition names repeated along the replicates.}

\item{replicates}{A vector of replicate names repeated along the conditions.}
}
\value{
An InteractionSet.
}
\description{
Parses interactions in \code{.hic} format and returns an InteractionSet
object.
}
\examples{
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.hic",
    "path/to/condition-1.replicate-2.hic",
    "path/to/condition-1.replicate-3.hic",
    "path/to/condition-2.replicate-1.hic",
    "path/to/condition-2.replicate-2.hic",
    "path/to/condition-2.replicate-3.hic"
)
# Replicate and condition of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)
# Resolution to select
binSize <- 500000
if (FALSE) {
    # Instantiation of data set
    hic.experiment <- parseHiC(
        paths,
        conditions = conditions,
        replicates = replicates,
        binSize = binSize
    )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{mergeInteractionSet}
\alias{mergeInteractionSet}
\title{Merge two InteractionSet objects}
\usage{
mergeInteractionSet(interactionSet1, interactionSet2, fill = NA)
}
\arguments{
\item{interactionSet1}{The first \code{\link{InteractionSet}}.}

\item{interactionSet2}{The second \code{\link{InteractionSet}}.}

\item{fill}{Fill missing values with this.}
}
\value{
The merged \code{\link{InteractionSet}}.
}
\description{
Merge two different \code{\link{InteractionSet}}.
}
\examples{
path <- system.file("extdata", "hicsample_21.cool", package = "HiCParser")
object1 <- parseCool(path, conditions = 1, replicates = 1)
# Creating an object with a different condition
object2 <- parseCool(path, conditions = 2, replicates = 1)
objectMerged <- mergeInteractionSet(object1, object2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabularParsers.R
\name{parseTabular}
\alias{parseTabular}
\title{Parser for tabular data}
\usage{
parseTabular(path, sep = '\t')
}
\arguments{
\item{path}{A path to a tabular file.}

\item{sep}{The separator of the tabular file. Default to tabulation.}
}
\value{
An InteractionSet object.
}
\description{
Read the file, produce an \code{InteractionSet} object.
}
\details{
Accepts a tabular file with \code{chromosome}, \code{position 1},
\code{position 2}, and multiple replicate columns listing interaction counts.
Null interactions do not have to be listed. Values must be separated by
tabulations. The header must be
\code{chromosome position 1 position 2 x.y x.y x.y ...} with \code{x}
replaced by condition names and \code{y} replaced by replicate names.
}
\examples{
path <- system.file("extdata", "hicsample_21.tsv", package = "HiCParser")
object <- parseTabular(path, sep = "\t")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HiCParser-package.R
\docType{package}
\name{HiCParser-package}
\alias{HiCParser}
\alias{HiCParser-package}
\title{A multiple format Hi-C data parser}
\description{
This package is a parser to import HiC data into R. It accepts several type of data: tabular files, Cooler `.cool` or `.mcool` files, Juicer `.hic` files or HiC-Pro `.matrix` and `.bed` files. The HiC data can be several files, for several replicates and conditions. The data is formated in an InteractionSet object.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/emaigne/HiCParser}
  \item Report bugs at \url{https://github.com/emaigne/HiCParser/issues}
}

}
\author{
\strong{Maintainer}: Maigné Élise \email{elise.maigne@inrae.fr}

Authors:
\itemize{
  \item Zytnicki Matthias \email{matthias.zytnicki@inrae.fr}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coolParser.R
\name{parseCool}
\alias{parseCool}
\title{Parser for data in cool format}
\usage{
parseCool(paths, binSize = NA, conditions, replicates)
}
\arguments{
\item{paths}{A vector of paths to \code{.cool} or \code{.mcool} files.}

\item{binSize}{The resolution (span of each position in number of bases). Optionally
provided to select the appropriate resolution in \code{.mcool} files.
Defaults to NULL.}

\item{conditions}{A vector of condition names repeated along the replicates.}

\item{replicates}{A vector of replicate names repeated along the conditions.}
}
\value{
An InteractionSet.
}
\description{
Parses interactions in \code{.cool} or \code{.mcool} format and returns
an InteractionSet object.
}
\details{
To read \code{.cool} of \code{.mcool} files, the \code{rhdf5} package is required.
Please install it before running the function.
}
\examples{
# EXAMPLE FOR .cool FORMAT
# Path to each file
pathsCool <- c(
    "path/to/condition-1.replicate-1.cool",
    "path/to/condition-1.replicate-2.cool",
    "path/to/condition-1.replicate-3.cool",
    "path/to/condition-2.replicate-1.cool",
    "path/to/condition-2.replicate-2.cool",
    "path/to/condition-2.replicate-3.cool"
)
# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)
if (FALSE) {
    library(rhdf5)
    object <- parseCool(
        paths,
        conditions = conditions,
        replicates = replicates
    )
}

# EXAMPLE FOR .mcool FORMAT
# Resolution
binSize <- 500000
# Path to each file
paths <- c(
    "path/to/condition-1.replicate-1.mcool",
    "path/to/condition-1.replicate-2.mcool",
    "path/to/condition-1.replicate-3.mcool",
    "path/to/condition-2.replicate-1.mcool",
    "path/to/condition-2.replicate-2.mcool",
    "path/to/condition-2.replicate-3.mcool"
)
# Condition and replicate of each file. Can be names instead of numbers.
conditions <- c(1, 1, 1, 2, 2, 2)
replicates <- c(1, 2, 3, 1, 2, 3)
if (FALSE) {
    # Instantiation of data set
    library(rhdf5)
    object <- parseCool(
        paths,
        conditions = conditions,
        replicates = replicates,
        binSize = binSize
    )
}
}
